package uk.co.his.test.packaging.junit;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;

public class JdbcUtils {

	private JdbcUtils() {
	}

	public static void showWarnings(Connection c) throws SQLException {
		SQLWarning w = c.getWarnings();
		while (w != null) {
			String s = w.getMessage();
			if ((s.indexOf("Changed language setting") == -1)
					&& (s.indexOf("Changed database context") == -1)
					&& (s.indexOf("Database changed") == -1)
					&& (s.indexOf("Language changed") == -1)) {
				System.out.println(s);
			}
			w = w.getNextWarning();
		}
		c.clearWarnings();
	}

	public static Connection getConnection(String driver, String url,
			boolean useTransaction) throws SQLException, ClassNotFoundException {

		Class.forName(driver);

		Connection c = DriverManager.getConnection(url);
		if (useTransaction) {
			c.setAutoCommit(false);
		}
		showWarnings(c);
		return c;
	}

	public static Connection getConnection(String url, boolean useTransaction)
			throws SQLException, ClassNotFoundException {
		Connection c = DriverManager.getConnection(url);
		if (useTransaction) {
			c.setAutoCommit(false);
		}
		showWarnings(c);
		return c;
	}

	public static void executeUpdate(Connection c, String command) throws SQLException {
		Statement s = c.createStatement();
		@SuppressWarnings("unused")
		int result = s.executeUpdate(command);
		s.close();
	}

	public static void dropTable(Connection c, String tableName) throws SQLException {
		executeUpdate(c, "DROP TABLE " + tableName);
	}

	public static void createTable(Connection c, String tableName, String details)
			throws SQLException {
		String command = "CREATE TABLE " + tableName + "(" + details + ")";
		executeUpdate(c, command);
	}

	public static void dropIndex(Connection c, String indexName, String tableName)
			throws SQLException {
		executeUpdate(c, "DROP INDEX " + indexName + "." + tableName);
	}

	public static void createUniqueIndex(Connection c, String indexName, String tableName,
			String tableDetails) throws SQLException {
		executeUpdate(c, "CREATE UNIQUE INDEX " + indexName + " ON " + tableName
				+ tableDetails);
	}

}
