package uk.co.his.test.packaging.junit;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.SystemException;
import javax.transaction.UserTransaction;

import org.jboss.ejb.client.EJBClient;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import uk.co.his.test.ssb4.AppException4;
import uk.co.his.test.ssb4.SSB4Remote;
import uk.co.his.test.ssb5.AppException;
import uk.co.his.test.ssb5.SSB5Remote;


public class TestClient {

	private static InitialContext initialContext;

	public static InitialContext getInitialContext() throws NamingException {
		if (initialContext == null) {
			final Hashtable jndiProperties = new Hashtable();
	        jndiProperties.put(Context.URL_PKG_PREFIXES, "org.jboss.ejb.client.naming");
	        initialContext = new InitialContext(jndiProperties);
		}
		return initialContext;
	}

    @Test
	public void testSSBwithXA() throws NamingException, AppException4, AppException, NotSupportedException, SystemException, SecurityException, IllegalStateException, RollbackException, HeuristicMixedException, HeuristicRollbackException { 

    	final String rowKey = "TEST";
    	final int increment = 1;
    	int expectedVal = increment;
		UserTransaction utx = null;
		InitialContext ctx = getInitialContext();

		utx = EJBClient.getUserTransaction("hath");
		utx.setTransactionTimeout(900);
		utx.begin();
        
		SSB4Remote bean = (SSB4Remote) ctx
				.lookup("ejb:JBTest4EAR/JBTest4/SSB4!uk.co.his.test.ssb4.SSB4Remote");
		SSB5Remote bean2 = (SSB5Remote) ctx
				.lookup("ejb:JBTest5EAR/JBTest5/SSB5!uk.co.his.test.ssb5.SSB5Remote");
								    
		System.out.println(bean.hello("Ping: Bean1"));
		System.out.println(bean2.hello("Ping: Bean2"));
		
		System.out.println("Add 100");
		int bean1Val = bean.add(rowKey, 100);
		int bean2Val = bean2.add(rowKey, 100);
		
		System.out.println("	Bean1: " + bean1Val);
		System.out.println("	Bean2: " + bean2Val);
		Assert.assertTrue("Beans should be using different databases,  what one does should not effect the other", bean1Val==bean2Val);
		System.out.println("rollback");
		utx.rollback();
		
        utx.begin();
		System.out.println("Add 1");
		bean1Val = bean.add(rowKey, increment);
		bean2Val = bean2.add(rowKey, increment);
		System.out.println("	Bean1: " + bean1Val);
		System.out.println("	Bean2: " + bean2Val);
		Assert.assertTrue("Beans should be using different databases,  what one does should not effect the other", bean1Val==bean2Val);
		Assert.assertTrue("Previous Txn rolledback, new value should be 1", bean1Val==expectedVal);
		Assert.assertTrue("Previous Txn rolledback, new value should be 1", bean2Val==expectedVal);
		System.out.println("commit");
        utx.commit();
        expectedVal += increment;
        
        utx.begin();
		System.out.println("Add 1");
		bean1Val = bean.add(rowKey, increment);
		bean2Val = bean2.add(rowKey, increment);
		System.out.println("	Bean1: " + bean1Val);
		System.out.println("	Bean2: " + bean2Val);
		Assert.assertTrue("Beans should be using different databases,  what one does should not effect the other", bean1Val==bean2Val);
		Assert.assertTrue("Previous Txn rolledback, new value should be 1", bean1Val==expectedVal);
		Assert.assertTrue("Previous Txn rolledback, new value should be 1", bean2Val==expectedVal);
		System.out.println("rollback");
        utx.rollback();
        //Rolled back increment dioesn't happen
        //expectedVal += increment;
        
        utx.begin();
		System.out.println("Add 1");
		bean1Val = bean.add(rowKey, increment);
		bean2Val = bean2.add(rowKey, increment);
		System.out.println("	Bean1: " + bean1Val);
		System.out.println("	Bean2: " + bean2Val);
		Assert.assertTrue("Beans should be using different databases,  what one does should not effect the other", bean1Val==bean2Val);
		Assert.assertTrue("Previous Txn rolledback, new value should be 1", bean1Val==expectedVal);
		Assert.assertTrue("Previous Txn rolledback, new value should be 1", bean2Val==expectedVal);
		System.out.println("commit");
        utx.commit();
        expectedVal += increment;
        
        utx.begin();
		System.out.println("Add 1");
		bean1Val = bean.add(rowKey, increment);
		bean2Val = bean2.add(rowKey, increment);
		System.out.println("	Bean1: " + bean1Val);
		System.out.println("	Bean2: " + bean2Val);
		Assert.assertTrue("Beans should be using different databases,  what one does should not effect the other", bean1Val==bean2Val);
		Assert.assertTrue("Previous Txn rolledback, new value should be 1", bean1Val==expectedVal);
		Assert.assertTrue("Previous Txn rolledback, new value should be 1", bean2Val==expectedVal);
		System.out.println("commit");
        utx.commit();

        System.out.println("Dones!");
	}
	
	//private static final String StdDbConnectionString = "jdbc:sqlserver://localhost:1433;databaseName=JBossTests;integratedSecurity=true";
	private static final String StdDbConnectionString = "jdbc:sqlserver://localhost:1433;databaseName=JBossTests;user=db-user;password=password";
	
	public static void testCreateDb() throws ClassNotFoundException, SQLException
	{
		Connection c = JdbcUtils.getConnection(StdDbConnectionString, false);
		JdbcUtils.createTable(c, "TaggedIntegerValue", "Tag VARCHAR(255) NOT NULL, Val INTEGER NOT NULL");
		JdbcUtils.createUniqueIndex(c, "TaggedIntegerValue", "TaggedIntegerValue", "(Tag)");
		c.close();
	}
	
	
	public static void testDropDb() throws ClassNotFoundException, SQLException
	{
		Connection c = JdbcUtils.getConnection(StdDbConnectionString, false);
		try {
			JdbcUtils.dropTable(c, "TaggedIntegerValue");
		} catch (Exception e) {
		}
		c.close();
	}

	private static final String StdDbConnectionString2 = "jdbc:sqlserver://localhost:1433;databaseName=JBossTests2;user=db-user;password=password";
	
	public static void testCreateDb2() throws ClassNotFoundException, SQLException
	{
		Connection c = JdbcUtils.getConnection(StdDbConnectionString2, false);
		JdbcUtils.createTable(c, "TaggedIntegerValue", "Tag VARCHAR(255) NOT NULL, Val INTEGER NOT NULL");
		JdbcUtils.createUniqueIndex(c, "TaggedIntegerValue", "TaggedIntegerValue", "(Tag)");
		c.close();
	}
	
	
	public static void testDropDb2() throws ClassNotFoundException, SQLException
	{
		Connection c = JdbcUtils.getConnection(StdDbConnectionString2, false);
		try {
			JdbcUtils.dropTable(c, "TaggedIntegerValue");
		} catch (Exception e) {
		}
		c.close();
	}
	
	@BeforeClass
	public static void CreateCleanDbs() throws ClassNotFoundException, SQLException {
		testDropDb();
		testDropDb2();
		testCreateDb();
		testCreateDb2();
	}
}
